# Vendored dependencies

This folder includes a local copy of <https://github.com/caltechlibrary/pybtex-apa7-style>, which is a patched fork of <https://github.com/cproctor/pybtex-apa7-style>, which in turn is a fork of the package <https://github.com/naeka/pybtex-apa-style>.

The folder you see here exists because we want to use PyPI (so that IGA's users can simply do `pip install iga`), but face the following problem:

1. We need the package `pybtex-apa7-style`. This package cannot merely be put in a subdirectory and loaded using an `import` statement; the package _must_ be installed using Python's setuptools (which is done by a typical `pip` installation) because it takes advantage of the [Python plugin](https://packaging.python.org/en/latest/guides/creating-and-discovering-plugins/) mechanism to register a plugin that the [Pybtex](https://pybtex.org) uses. (The package `pybtex-apa-style` would suit IGA as well, but it has the same bugs described in point #2, so we may as well work with the more up-to-date `pytex-apa7-style`.)
2. The original version of `pybtex-apa7-style` (at <https://github.com/cproctor/pybtex-apa7-style>) has at least two bugs that affect us, one of which is a show-stopper. They are simple bugs, and issues have been filed with the original project repository on GitHub, but as of the time of this writing (April 2023), the bugs still exist. So, I made a fork (at <https://github.com/caltechlibrary/pybtex-apa7-style>) and patched the bugs.
3. During IGA's installation, we need to load our patched version of `pybtex-apa7-style`, which means we cannot simply write a line with "`pybtex-apa7-style`" in our `requirements.txt` file but rather must write a full direct reference to our copy of the package. Normally, the straightforward way to reference a package stored in a repo on GitHub would be to write a line such as `pybtex-apa7-style@git+https://github.com/caltechlibrary/pybtex-apa7-style.git` in the requirements file. And this would be fine; `pip` handles it without trouble, and that would be the end of the story, except ...
4. ... PyPI will reject packages that include a direct reference in their `requirements.txt`. PyPI will return the error `400 Invalid value for requires_dist. Error: Can't have direct dependency`. (The restriction is actually a sensible security measure.)

So we're stuck. Our choices come down to: (a) release our patched copy of `pybtex-apa7-style` on PyPI, or (b) vendor the package in a way that's compatible with both `pip` and PyPI's restrictions. Approach (a) would be simpler but probably lead to more confusion and maintenance in the future, so (b) is the approach taken.
